/*
 * Copyright (c) 1997 by Mark Hill, James Larus, and David Wood for
 * the Wisconsin Wind Tunnel Project.
 *
 * ALL RIGHTS RESERVED.
 * 
 * This software is furnished under a license and may be used and
 * copied only in accordance with the terms of such license and the
 * inclusion of the above copyright notice.  This software or any other
 * copies thereof or any derivative works may not be provided or
 * otherwise made available to any other persons.  Title to and ownership
 * of the software is retained by Mark Hill, James Larus, and David Wood.
 * Any use of this software must include the above copyright notice.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS".  THE LICENSOR MAKES NO
 * WARRANTIES ABOUT ITS CORRECTNESS OR PERFORMANCE.
 *
 * This file is part of the Blizzard source distribution
 * originally written by Babak Falsafi, Alvin Lebeck, Steven
 * K. Reinhardt, and Yannis Schoinas and subsequently maintained 
 * by Babak Falsafi, Chris Lukas, Steven K. Reinhardt, Yannis Schoinas
 * and Brian Toonen.
 */
#ifndef _addr_inlines_h
#define _addr_inlines_h

#include "../master/tppi_vm_decl.h"
#include "../master/tppi_addr_space.h"

#include "inline.h"

INLINE 
unsigned page_num(void *va)
{
    return ((unsigned)va >> TPPI_PAGE_SHIFT);
}

#define page_align page_addr

INLINE 
void *page_addr(void *va)
{
    return (void *)((unsigned)va & ~(TPPI_PAGE_SIZE-1));
}

INLINE 
void *next_page_addr(void *va)
{
    return (char *)page_addr((char *)va-1) + TPPI_PAGE_SIZE;
}

INLINE 
unsigned page_offs(void *va)
{
    return ((unsigned)va & (TPPI_PAGE_SIZE-1));
}

INLINE 
unsigned blk_num(void *va)
{
    return (page_offs(va) >> TPPI_TAG_BLK_SHIFT);
}

INLINE 
void *blk_align(void *va)
{
    return (void *)((unsigned)va & ~(TPPI_TAG_BLK_SIZE-1));
}

INLINE
void *blk_align_to_size(void *va, unsigned int blk_size)
{
    return (void *)((unsigned)va & ~(blk_size-1));
}

INLINE 
unsigned blk_offs(void *va)
{
    return ((unsigned)va & (TPPI_TAG_BLK_SIZE-1));
}

#endif
