/*
 * Copyright (c) 1997 by Mark Hill, James Larus, and David Wood for
 * the Wisconsin Wind Tunnel Project.
 *
 * ALL RIGHTS RESERVED.
 * 
 * This software is furnished under a license and may be used and
 * copied only in accordance with the terms of such license and the
 * inclusion of the above copyright notice.  This software or any other
 * copies thereof or any derivative works may not be provided or
 * otherwise made available to any other persons.  Title to and ownership
 * of the software is retained by Mark Hill, James Larus, and David Wood.
 * Any use of this software must include the above copyright notice.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS".  THE LICENSOR MAKES NO
 * WARRANTIES ABOUT ITS CORRECTNESS OR PERFORMANCE.
 *
 * This file is part of the Blizzard source distribution
 * originally written by Babak Falsafi, Alvin Lebeck, Steven
 * K. Reinhardt, and Yannis Schoinas and subsequently maintained 
 * by Babak Falsafi, Chris Lukas, Steven K. Reinhardt, Yannis Schoinas
 * and Brian Toonen.
 */
#ifndef _local_lock_h
#define _local_lock_h

#include "../master/types.h"
#include "../master/util_inlines.h"

#define LOCKED   0xff
#define UNLOCKED 0x0

typedef volatile Uint8 LocalLock;
typedef LocalLock atomic_sequence_t;

#ifdef __cplusplus
extern "C" {
#endif __cplusplus

#define start_atomic_sequence _NI_start_atomic_sequence
#define end_atomic_sequence _NI_end_atomic_sequence

INLINE 
void acquireLocalLock(LocalLock *l)
{
    /* atomically reads *l and writes 0xff */
    cbarrier();
    while (ldstub(l) == LOCKED);
    cbarrier();
}

INLINE 
void releaseLocalLock(LocalLock *l)
{
    cbarrier();
    *l = UNLOCKED;
    cbarrier();
}

#define init_atomic_sequence	_NI_init_atomic_sequence

INLINE 
void init_atomic_sequence(atomic_sequence_t *seq)
{
    seq = UNLOCKED;
}
 
INLINE 
void start_atomic_sequence(atomic_sequence_t *seq_id)
{
    acquireLocalLock(seq_id);
}

INLINE 
void end_atomic_sequence(atomic_sequence_t *seq_id)
{
    releaseLocalLock(seq_id);
}

#ifdef __cplusplus
};
#endif __cplusplus

#endif _local_lock_h
