/*
 * Copyright (c) 1997 by Mark Hill, James Larus, and David Wood for
 * the Wisconsin Wind Tunnel Project.
 *
 * ALL RIGHTS RESERVED.
 * 
 * This software is furnished under a license and may be used and
 * copied only in accordance with the terms of such license and the
 * inclusion of the above copyright notice.  This software or any other
 * copies thereof or any derivative works may not be provided or
 * otherwise made available to any other persons.  Title to and ownership
 * of the software is retained by Mark Hill, James Larus, and David Wood.
 * Any use of this software must include the above copyright notice.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS".  THE LICENSOR MAKES NO
 * WARRANTIES ABOUT ITS CORRECTNESS OR PERFORMANCE.
 *
 * This file is part of the Stache source distribution
 * originally written by Alvin Lebeck and Steven K. Reinhardt and 
 * subsequently maintained by Babak Falsafi, Steven K. Reinhardt and 
 * Yannis Schoinas.
 */

#ifndef _stache_h
#define _stache_h
#include "addr_inlines.h"	/* for page_offs() */
#include "tppi_vm.h"

#define STACHE_BLKS_PER_PAGE	(TPPI_PAGE_SIZE/STACHE_BLK_SIZE)

#define stache_blk_num(va)	(page_offs(va) / STACHE_BLK_SIZE)
#define stache_blk_align(va)    ((void *)((unsigned)va & ~(STACHE_BLK_SIZE-1)))

/*
 * Home page type declarations
 */

#define	NUM_PTRS		6
#define STACHE_MAX_NODES	32	/* since we keep bitvec in an int */

typedef struct HomeBlkInfo_str {
    union  {
        unsigned bit_vec;
        unsigned char ptr_vec[4];
    } dir_un;
    unsigned char more_ptrs[2];
    unsigned char ctr;
    unsigned char state;

#ifdef STACHE_UPDATE
    unsigned char update_ctr;
    unsigned char update_src;
#endif STACHE_UPDATE

} HomeBlkInfo;


typedef struct HomePageInfo_str {
    HomeBlkInfo blk_info[STACHE_BLKS_PER_PAGE];
} HomePageInfo;

/*
 * home.c handlers
 */

void _NI_GetRO_handler();
void _NI_GetRW_handler();
void _NI_Upgrade_handler();
void _NI_Put_Data_handler();
void _NI_Put_No_Data_handler();

void _NI_Home_rd_acc_fault();
void _NI_Home_wr_acc_inv_fault();
void _NI_Home_wr_acc_ro_fault();

void _NI_null_fault();

/*
 * Stache page type declarations
 */

typedef unsigned char StacheBlkInfo;

typedef struct StachePageInfo_str {
    StacheBlkInfo blk_info[STACHE_BLKS_PER_PAGE];

#ifdef STACHE_PAGE_STATS

    char touched[STACHE_BLKS_PER_PAGE];  /* has this block ever been touched */
    int max_active_blks; /* max number of active blocks */
    int cur_active_blks; /* current number of active blocks */

#endif STACHE_PAGE_STATS

}StachePageInfo;

#define BLK_FLUSH	8

enum StacheBlkState {
    Stache_Blk_Invalid,
    Stache_Blk_Present_RO,
    Stache_Blk_Present_RW,
    Stache_Blk_Get_RO,
    Stache_Blk_Get_RW,
    Stache_Blk_Upgrade,
    Stache_Blk_Get_RO_Flush  = Stache_Blk_Get_RO | BLK_FLUSH,
    Stache_Blk_Get_RW_Flush  = Stache_Blk_Get_RW | BLK_FLUSH,
    Stache_Blk_Upgrade_Flush = Stache_Blk_Upgrade | BLK_FLUSH
};

/*
 * stache.c handlers
 */

void _NI_GetRW_Resp_handler();
void _NI_Upgrade_Ack_handler();
void _NI_GetRO_Resp_handler();
void _NI_PutRO_Req_handler();
void _NI_PutRW_Req_handler();
void _NI_Nack_handler();
void _NI_flushRO_retry();
void _NI_flushRW_retry();

void _NI_Stache_rd_acc_fault();
void _NI_Stache_wr_acc_inv_fault();
void _NI_Stache_wr_acc_ro_fault();


#define	STACHE_TR_BASE		TRACE_PROT1
#define STACHE_TR_BAFS		TRACE_PROT2
#define STACHE_TR_NACKS		TRACE_PROT3
#define STACHE_TR_DATA		TRACE_PROT4


#define TRACE_DATA(trace_tag,va)			\
{							\
    int *ip = (int *)blk_align(va);			\
    TRACE4(trace_tag, "    %08x %08x %08x %08x\n",	\
	   ip[0], ip[1], ip[2], ip[3]);			\
    ip += 4;						\
    TRACE4(trace_tag, "    %08x %08x %08x %08x\n",	\
	   ip[0], ip[1], ip[2], ip[3]);			\
}

void stache_flush(void *va);

#endif
