/*
 * Copyright (c) 1997 by Mark Hill, James Larus, and David Wood for
 * the Wisconsin Wind Tunnel Project.
 *
 * ALL RIGHTS RESERVED.
 * 
 * This software is furnished under a license and may be used and
 * copied only in accordance with the terms of such license and the
 * inclusion of the above copyright notice.  This software or any other
 * copies thereof or any derivative works may not be provided or
 * otherwise made available to any other persons.  Title to and ownership
 * of the software is retained by Mark Hill, James Larus, and David Wood.
 * Any use of this software must include the above copyright notice.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS".  THE LICENSOR MAKES NO
 * WARRANTIES ABOUT ITS CORRECTNESS OR PERFORMANCE.
 *
 * This file is part of the Blizzard source distribution
 * originally written by Babak Falsafi, Alvin Lebeck, Steven
 * K. Reinhardt, and Yannis Schoinas and subsequently maintained 
 * by Babak Falsafi, Chris Lukas, Steven K. Reinhardt, Yannis Schoinas
 * and Brian Toonen.
 */
#ifndef _tat_lock_h
#define _tat_lock_h

#ifdef __cplusplus
extern "C" {
#endif

#include "types.h"
#include "util_inlines.h"
#include "local_lock.h"

/* Msg passing locks for PARMACS */

typedef struct Tat_Lock *LockPtr;

struct Tat_Lock
{
    Uint8 lock;   /* lock node */
    Uint8 pad[31];
};

/*
 * Declare a lock.
 * This pointer can be in private or shared memory as long as only one node
 * does the BLZ_LOCKINIT(lockname) and all nodes get the value from that.
 */
#define TAT_LOCKDEC(lockname)  	    LockPtr lockname;

/* Initialize a lock */
#define TAT_LOCKINIT(lockname)                           \
      lockname = (LockPtr)shm_malloc(struct Tat_Lock);

/* Initialize a lock, using sbrk instead of malloc.  This is for malloc. */
#define TAT_SBRK_LOCKINIT(lockname)                      \
      lockname = (LockPtr)shm_sbrk(struct Tat_Lock);

void tat_lock(LockPtr);
void tat_unlock(LockPtr);

#ifdef __cplusplus
}
#endif

#endif _tat_lock_h
