/*
 * Copyright (c) 1997 by Mark Hill, James Larus, and David Wood for
 * the Wisconsin Wind Tunnel Project.
 *
 * ALL RIGHTS RESERVED.
 * 
 * This software is furnished under a license and may be used and
 * copied only in accordance with the terms of such license and the
 * inclusion of the above copyright notice.  This software or any other
 * copies thereof or any derivative works may not be provided or
 * otherwise made available to any other persons.  Title to and ownership
 * of the software is retained by Mark Hill, James Larus, and David Wood.
 * Any use of this software must include the above copyright notice.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS".  THE LICENSOR MAKES NO
 * WARRANTIES ABOUT ITS CORRECTNESS OR PERFORMANCE.
 *
 * This file is part of the Blizzard source distribution
 * originally written by Babak Falsafi, Alvin Lebeck, Steven
 * K. Reinhardt, and Yannis Schoinas and subsequently maintained 
 * by Babak Falsafi, Chris Lukas, Steven K. Reinhardt, Yannis Schoinas
 * and Brian Toonen.
 */
#ifndef _blk_acc_h
#define _blk_acc_h

#include "../master/tppi_vm.h" 

#define TPPI_change_blk_acc           _NI_TPPI_change_blk_acc
#define TPPI_change_blk_acc_and_get   _NI_TPPI_change_blk_acc_and_get
#define TPPI_change_blk_acc_and_put   _NI_TPPI_change_blk_acc_and_put
#define TPPI_get_blk_acc	      _NI_rd_blk_acc

#define hard_resume	_NI_hard_resume
#define soft_resume	_NI_soft_resume


#ifdef __cplusplus
extern "C" {
#endif

typedef enum 
{
    TPPI_Blk_No_Tag_Change,	/* no change; useful as function arg */
    TPPI_Blk_Validate_RO,	/* anything --> RO */
    TPPI_Blk_Validate_RW,	/* anything --> RW */
    TPPI_Blk_Upgrade_RW,	/* RO --> RW */
    TPPI_Blk_Downgrade_RO,	/* RW --> RO */
    TPPI_Blk_Invalidate,	/* anything --> inval */
    TPPI_Blk_Mark_Busy,		/* anything --> busy */
    TPPI_Blk_Invalid_To_Busy,	/* inval --> busy */
    TPPI_Blk_Busy_To_Invalid	/* busy --> inval */
} TPPI_BlkTagChange;


TPPI_BlkAccTag TPPI_get_blk_acc(void *va);

void TPPI_change_blk_acc(void *va, int len, TPPI_BlkTagChange chg);
void TPPI_change_blk_acc_and_get(void *va, int len, TPPI_BlkTagChange chg,
                                 void *from);
void TPPI_change_blk_acc_and_put(void *va, int len, TPPI_BlkTagChange chg,
                                 void *to);

void hard_resume(void *, int);
void soft_resume(void *);

#define TPPI_resume_va(a, b)	hard_resume(a, b)

#ifdef __cplusplus
}
#endif

#endif _blk_acc_h
