/*
 * Copyright (c) 1997 by Mark Hill, James Larus, and David Wood for
 * the Wisconsin Wind Tunnel Project.
 *
 * ALL RIGHTS RESERVED.
 * 
 * This software is furnished under a license and may be used and
 * copied only in accordance with the terms of such license and the
 * inclusion of the above copyright notice.  This software or any other
 * copies thereof or any derivative works may not be provided or
 * otherwise made available to any other persons.  Title to and ownership
 * of the software is retained by Mark Hill, James Larus, and David Wood.
 * Any use of this software must include the above copyright notice.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS".  THE LICENSOR MAKES NO
 * WARRANTIES ABOUT ITS CORRECTNESS OR PERFORMANCE.
 *
 * This file is part of the Blizzard source distribution
 * originally written by Babak Falsafi, Alvin Lebeck, Steven
 * K. Reinhardt, and Yannis Schoinas and subsequently maintained 
 * by Babak Falsafi, Chris Lukas, Steven K. Reinhardt, Yannis Schoinas
 * and Brian Toonen.
 */
#ifndef _blizzard_msg_h
#define _blizzard_msg_h

#include "../master/internals.h"

#ifdef XXX /* Fix me: I don't want to worry about Guhan's libs now */
#define TPPI_channel_send         	  _NI_send_bulk_data
#define TPPI_channel_send_s               _NI_send_bulk_data_s
#define TPPI_get_dst_channel_user_ptr     _NI_TPPI_get_dst_channel_user_ptr
#define TPPI_is_channel_reset		  _NI_TPPI_is_channel_reset
#define TPPI_set_dst_channel_user_ptr	  _NI_TPPI_set_dst_channel_user_ptr
#define TPPI_reset_channel_notify	  _NI_TPPI_reset_channel_notify
#define TPPI_is_channel_ready		  _NI_TPPI_is_channel_ready
#define TPPI_destroy_channel_src	  _NI_TPPI_destroy_channel_src
#define TPPI_get_src_channel_user_ptr	  _NI_TPPI_get_src_channel_user_ptr
#define TPPI_is_channel_estd		  _NI_TPPI_is_channel_estd
#define TPPI_set_channel_dst_notify	  _NI_TPPI_set_channel_dst_notify
#define TPPI_set_channel_src		  _NI_TPPI_set_channel_src
#define TPPI_reset_channel		  _NI_TPPI_reset_channel
#define TPPI_set_src_channel_user_ptr	  _NI_TPPI_set_src_channel_user_ptr
#define TPPI_set_channel_dst		  _NI_TPPI_set_channel_dst
#define TPPI_destroy_channel_dst	  _NI_TPPI_destroy_channel_dst
#endif


#define TPPI_send         _NI_send
#define TPPI_send_W       _NI_send_W
#define TPPI_send_WW      _NI_send_WW
#define TPPI_send_WWW     _NI_send_WWW
#define TPPI_send_WWWW    _NI_send_WWWW
#define TPPI_send_WWWWW   _NI_send_WWWWW
#define TPPI_send_WWWWWW  _NI_send_WWWWWW
#define TPPI_send_WWWWWWW _NI_send_WWWWWWW

#define TPPI_send_WBa		_NI_send_WBa
#define TPPI_send_WF		_NI_send_WF
#define TPPI_send_WR		_NI_send_WR
#define TPPI_send_WBs		_NI_send_WBs
#define TPPI_send_WRs		_NI_send_WRs
#define TPPI_send_WWBa		_NI_send_WWBa
#define TPPI_send_WWF		_NI_send_WWF
#define TPPI_send_WWR		_NI_send_WWR
#define TPPI_send_WWBs		_NI_send_WWBs
#define TPPI_send_WWRs		_NI_send_WWRs
#define TPPI_send_WWWBa		_NI_send_WWWBa
#define TPPI_send_WWWR		_NI_send_WWWR
#define TPPI_send_WWWBs		_NI_send_WWWBs
#define TPPI_send_WWWRs		_NI_send_WWWRs
#define TPPI_send_WWWBaR	_NI_send_WWWBaR
#define TPPI_send_WWWWBa	_NI_send_WWWWBa
#define TPPI_send_WWWWF		_NI_send_WWWWF
#define TPPI_send_WWWWR		_NI_send_WWWWR
#define TPPI_send_WWWWBs	_NI_send_WWWWBs
#define TPPI_send_WWWWRs	_NI_send_WWWWRs

#define TPPI_recv_W  _NI_recv_W
#define TPPI_recv_Ba _NI_recv_Ba
#define TPPI_recv_R  _NI_recv_R

/*
 * Tempest interface constant, to indicate maximum message data
 * payload in bytes.  Does not include handler PC, so we must subtract
 * one word.
 */
#define TPPI_MAX_AM_BYTES	((MAX_MSG_WORDS - 1) * 4)

#define TPPI_MAX_CHANNEL_BYTES  1024*1024

#include "../master/tppi_blk_acc.h"

/*
 * Channel functions
 */

typedef void (*TPPI_ChannelHandlerPtr)(int, int);

int TPPI_set_channel_src(int dest, TPPI_ChannelHandlerPtr pc);
void TPPI_destroy_channel_src(int dest, int channel);

int TPPI_set_channel_dst(int dest, int chn, void *buffer, int words,
			 TPPI_ChannelHandlerPtr pc);
void TPPI_destroy_channel_dst(int dest, int channel);
void TPPI_reset_channel(int src, int channel_id);

void TPPI_channel_send(int dest, int channel_id, int *buffer, int words);
void TPPI_channel_send_s(int dest, int channel_id, int *buffer, int words, 
			 int *rbuffer);

void TPPI_reset_channel_notify(int src, int channel_id);
int TPPI_is_channel_reset(int dst, int channel);
int TPPI_is_channel_ready(int dst, int channel);
int TPPI_is_channel_estd(int dst, int channel);
int TPPI_set_channel_dst_notify(int src, int channel, void *buffer,
				int bytes, TPPI_ChannelHandlerPtr recvfn);

void TPPI_set_src_channel_user_ptr(int dst, int channel, void *);
void *TPPI_get_src_channel_user_ptr(int dst, int channel);
void TPPI_set_dst_channel_user_ptr(int src, int channel, void *);
void *TPPI_get_dst_channel_user_ptr(int src, int channel);

/* Small messages */
void TPPI_send(int dest, void *pc);
void TPPI_send_W(int dest, void *pc, int a1);
void TPPI_send_WW(int dest, void *pc, int a1, int a2);
void TPPI_send_WWW(int dest, void *pc, int a1, int a2, int a3);
void TPPI_send_WWWW(int dest, void *pc, int a1, int a2, int a3, int a4);
void TPPI_send_WWWWW(int dest, void *pc, int a1, int a2, int a3, int a4, int a5);

/* Large messages :
 * Ba is used for blocks aligned to a len boundary 
 * F is used to forward something from the recv to the send queue
 * R is used for unaligned regions (no access change)
 */

/* one words plus a block/region */
void TPPI_send_WBa(int dest, void *pc, int a1,
		   int *ptr, int len, TPPI_BlkTagChange chg);
void TPPI_send_WR(int dest, void *pc, int a1, int *ptr, int len);
void TPPI_send_WBs(int dest, void *pc, int a1,
		   int *ptr, int len, TPPI_BlkTagChange chg,
		   int *rptr, TPPI_BlkTagChange rchg);
void TPPI_send_WRs(int dest, void *pc, int a1, int *ptr, int len, int *rptr);

/* two words plus a block/region */
void TPPI_send_WWBa(int dest, void *pc, int a1, int a2,
		    int *ptr, int len, TPPI_BlkTagChange chg);
void TPPI_send_WWR(int dest, void *pc, int a1, int a2, int *ptr, int len);
void TPPI_send_WWBs(int dest, void *pc, int a1, int a2,
		    int *ptr, int len, TPPI_BlkTagChange chg,
		    int *rptr, TPPI_BlkTagChange rchg);
void TPPI_send_WWRs(int dest, void *pc, int a1, int a2, int *ptr, int len, 
		    int *rptr);

/* three words plus a block/region */
void TPPI_send_WWWBa(int dest, void *pc, int a1, int a2, int a3,
		     int *ptr, int len, TPPI_BlkTagChange chg);
void TPPI_send_WWWR(int dest, void *pc, int a1, int a2, int a3,
		    int *ptr, int len);
void TPPI_send_WWWBs(int dest, void *pc, int a1, int a2, int a3,
		     int *ptr, int len, TPPI_BlkTagChange chg, 
		     int *rptr, TPPI_BlkTagChange rchg);
void TPPI_send_WWWRs(int dest, void *pc, int a1, int a2, int a3,
		     int *ptr, int len, int *rptr);

/* four words plus a block/region */
void TPPI_send_WWWWBa(int dest, void *pc, int a1, int a2, int a3, int a4,
		      int *ptr, int len, TPPI_BlkTagChange chg);
void TPPI_send_WWWWR(int dest, void *pc, int a1, int a2, int a3, int a4,
		     int *ptr, int len);
void TPPI_send_WWWWBs(int dest, void *pc, int a1, int a2, int a3, int a4,
		      int *ptr, int len, TPPI_BlkTagChange chg,
		      int *rptr, TPPI_BlkTagChange rchg);
void TPPI_send_WWWWRs(int dest, void *pc, int a1, int a2, int a3, int a4,
		      int *ptr, int len,
		      int *rptr);

int TPPI_recv_W();
void TPPI_recv_Ba(void *va, int len, TPPI_BlkTagChange chg);
void TPPI_recv_R(void *va, int len);

extern int TPPI_self_address;

#endif _blizzard_msg_h
