/*
 * Copyright (c) 1997 by Mark Hill, James Larus, and David Wood for
 * the Wisconsin Wind Tunnel Project.
 *
 * ALL RIGHTS RESERVED.
 * 
 * This software is furnished under a license and may be used and
 * copied only in accordance with the terms of such license and the
 * inclusion of the above copyright notice.  This software or any other
 * copies thereof or any derivative works may not be provided or
 * otherwise made available to any other persons.  Title to and ownership
 * of the software is retained by Mark Hill, James Larus, and David Wood.
 * Any use of this software must include the above copyright notice.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS".  THE LICENSOR MAKES NO
 * WARRANTIES ABOUT ITS CORRECTNESS OR PERFORMANCE.
 *
 * This file is part of the Blizzard source distribution
 * originally written by Babak Falsafi, Alvin Lebeck, Steven
 * K. Reinhardt, and Yannis Schoinas and subsequently maintained 
 * by Babak Falsafi, Chris Lukas, Steven K. Reinhardt, Yannis Schoinas
 * and Brian Toonen.
 */
#ifndef _tppi_vm_decl_h
#define _tppi_vm_decl_h

#include "mach.h"

/*
 * Types for fault handler function pointers
 */
typedef void (*TPPI_BlkAccFaultHandlerPtr)(void *va, void *user_ptr, int home);
typedef void (*TPPI_PageFaultHandlerPtr)(void *sfar, void *pc, int is_write);

typedef enum 
{
    TPPI_Blk_Writable,	/* 00; valid and writable */
    TPPI_Blk_ReadOnly,	/* 01; valid but not writable */
    TPPI_Blk_Invalid,	/* 10 */
    TPPI_Blk_Busy		/* 11 */
} TPPI_BlkAccTag;

#define TPPI_MAX_BLK_ACC_TAG	TPPI_Blk_Busy
#define TPPI_NUM_BLK_ACC_TAGS	(TPPI_MAX_BLK_ACC_TAG+1)

#define TPPI_NUM_PAGE_MODES	16
#define TPPI_MAX_PAGE_MODE	(TPPI_NUM_PAGE_MODES - 1)

#define TPPI_PAGE_SHIFT		MMU_PAGESHIFT
#define TPPI_PAGE_SIZE		(1 << TPPI_PAGE_SHIFT)

#define TPPI_TAG_BLK_SHIFT	LOG2_BLK_SIZE
#define TPPI_TAG_BLK_SIZE	(1 << TPPI_TAG_BLK_SHIFT)

typedef unsigned TPPI_NodeId;
typedef unsigned TPPI_PageMode;

typedef enum {
    ReadOnly,
    ReadWrite,
    ReadExec,
    ReadWriteExec,
    NoAccess		/* for BlizzardP */
} PageProt;

#endif _tppi_vm_decl_h
