/*
 * Copyright (c) 1997 by Mark Hill, James Larus, and David Wood for
 * the Wisconsin Wind Tunnel Project.
 *
 * ALL RIGHTS RESERVED.
 * 
 * This software is furnished under a license and may be used and
 * copied only in accordance with the terms of such license and the
 * inclusion of the above copyright notice.  This software or any other
 * copies thereof or any derivative works may not be provided or
 * otherwise made available to any other persons.  Title to and ownership
 * of the software is retained by Mark Hill, James Larus, and David Wood.
 * Any use of this software must include the above copyright notice.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS".  THE LICENSOR MAKES NO
 * WARRANTIES ABOUT ITS CORRECTNESS OR PERFORMANCE.
 *
 * This file is part of the Blizzard source distribution
 * originally written by Babak Falsafi, Alvin Lebeck, Steven
 * K. Reinhardt, and Yannis Schoinas and subsequently maintained 
 * by Babak Falsafi, Chris Lukas, Steven K. Reinhardt, Yannis Schoinas
 * and Brian Toonen.
 */
#ifndef _trace_h
#define _trace_h

#define trace_func	_NI_trace_func

#ifdef __cplusplus
extern "C" {
#endif __cplusplus

void init_trace(void);
void trace_func(char *,int,...);

int dump_trace(void);

int append_trace(void);

#ifdef __cplusplus
};
#endif __cplusplus

#define TRACE_MASK(i)	(1 << i)

#define	TRACE_MMU	TRACE_MASK(0)	/* 0x01 */
#define	TRACE_PKT	TRACE_MASK(1)	/* 0x02 */
#define	TRACE_TRAPS	TRACE_MASK(2)	/* 0x04 */
#define	TRACE_IPC	TRACE_MASK(3)	/* 0x08 */
#define	TRACE_IO	TRACE_MASK(4)	/* 0x10 */
#define	TRACE_PMACS	TRACE_MASK(5)	/* 0x20 */
#define	TRACE_BAR	TRACE_MASK(6)	/* 0x40 */
#define TRACE_LOCK	TRACE_MASK(7)	/* 0x80 */
#define	TRACE_MALLOC	TRACE_MASK(8)   /* 0x100 */
#define	TRACE_IMASK	TRACE_MASK(9)   /* 0x200 */
#define	TRACE_TAG  	TRACE_MASK(10)  /* 0x400 */
#define	TRACE_THREAD  	TRACE_MASK(11)  /* 0x800 */
#define	TRACE_IOMMU  	TRACE_MASK(12)  /* 0x1000 */

/*
 * Indices 16-31 are for the use of user-level protocols.
 */
#define	TRACE_PROT1	TRACE_MASK(16)
#define	TRACE_PROT2	TRACE_MASK(17)
#define	TRACE_PROT3	TRACE_MASK(18)
#define	TRACE_PROT4	TRACE_MASK(19)

#ifndef NO_TRACE

extern int blz_trace_mask;

#define TRACE(m,s)			\
do {					\
    if (blz_trace_mask&(m)) {		\
	trace_func(s,0);		\
    }					\
} while (0);

#define TRACE1(m,s,a)			\
do {					\
    if (blz_trace_mask&(m)) {		\
	trace_func(s,1,a);		\
    }					\
} while (0);

#define TRACE2(m,s,a,b)			\
do {					\
    if (blz_trace_mask&(m)) {		\
	trace_func(s,2,a,b);		\
    }					\
} while (0);

#define TRACE3(m,s,a,b,c)		\
do {					\
    if (blz_trace_mask&(m)) {		\
	trace_func(s,3,a,b,c);		\
    }					\
} while (0);

#define TRACE4(m,s,a,b,c,d)		\
do {					\
    if (blz_trace_mask&(m)) {		\
	trace_func(s,4,a,b,c,d);	\
    }					\
} while (0);

#define TRACE5(m,s,a,b,c,d,e)		\
do {					\
    if (blz_trace_mask&(m)) {		\
	trace_func(s,5,a,b,c,d,e);	\
    }					\
} while (0);

#else

#define TRACE(m,s)
#define TRACE1(m,s,a)
#define TRACE2(m,s,a,b)
#define TRACE3(m,s,a,b,c)
#define TRACE4(m,s,a,b,c,d)
#define TRACE5(m,s,a,b,c,d,e)

#endif

#endif
