/*
 * Copyright (c) 1997 by Mark Hill, James Larus, and David Wood for
 * the Wisconsin Wind Tunnel Project.
 *
 * ALL RIGHTS RESERVED.
 * 
 * This software is furnished under a license and may be used and
 * copied only in accordance with the terms of such license and the
 * inclusion of the above copyright notice.  This software or any other
 * copies thereof or any derivative works may not be provided or
 * otherwise made available to any other persons.  Title to and ownership
 * of the software is retained by Mark Hill, James Larus, and David Wood.
 * Any use of this software must include the above copyright notice.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS".  THE LICENSOR MAKES NO
 * WARRANTIES ABOUT ITS CORRECTNESS OR PERFORMANCE.
 *
 * This file is part of the Blizzard source distribution
 * originally written by Babak Falsafi, Alvin Lebeck, Steven
 * K. Reinhardt, and Yannis Schoinas and subsequently maintained 
 * by Babak Falsafi, Chris Lukas, Steven K. Reinhardt, Yannis Schoinas
 * and Brian Toonen.
 */
#ifndef _tppi_types_h
#define _tppi_types_h


#if !defined(__cplusplus) || !(defined(__GNUC__) && (__GNUC__ >= 2) && (__GNUC_MINOR__ >= 6))

  /* as of gcc 2.6, bool is a builtin type */

  typedef unsigned char bool;

# define true	((bool)1)
# define false	((bool)0)

#endif

#ifndef TRUE
#  define TRUE    1
#endif

#ifndef FALSE
#  define FALSE   0
#endif

#ifndef NULL
#  define NULL    0
#endif

#include "inline.h"

typedef unsigned char Uint8;
typedef unsigned short int Uint16;
typedef unsigned int Uint32;
typedef unsigned long long int Uint64;

typedef union {
    struct {
	Uint32 hi, lo;
    } by32;
    Uint64 by64;
} Uint64_32_Union;

#ifdef __cplusplus

inline
Uint32 lo32(Uint64 data)
{
    return (data & (((Uint64)1 << 32) - 1));
}

inline
Uint32 hi32(Uint64 data)
{
    return lo32(data >> 32);
}

#else

INLINE 
Uint32 lo32(Uint64 data)
{
    return (data & (((Uint64)1 << 32) - 1));
}

INLINE 
Uint32 hi32(Uint64 data)
{
    return lo32(data >> 32);
}

#endif __cplusplus

INLINE 
Uint64 hiLo32To64(Uint32 hi, Uint32 lo)
{
    Uint64_32_Union u;
    u.by32.hi = hi;
    u.by32.lo = lo;
    return u.by64;
}

typedef Uint32 VirtAddr;
typedef Uint64 PhysAddr;

typedef void (*FuncPtr)(void);
typedef bool (*SpinFuncPtr)(int);

#endif _tppi_types_h
